var BASIC_SHADER_FS = MultiLineString(function() {/*
precision mediump float;
uniform vec4 colour;
uniform sampler2D texture;

varying vec2 coords;

void main() {
    gl_FragColor = texture2D(texture, coords) + colour;
}
*/
});
      
var BASIC_SHADER_VS = MultiLineString(function() {/*
precision mediump float;
attribute vec3 position;
attribute vec2 texCoord;

varying vec2 coords;

uniform mat4 model;
uniform mat4 projection;

void main(void) {
    coords = texCoord;
    gl_Position = projection * model * vec4(position, 1.0);
}
*/
});

var SIMPLE_SHADER;

function Shader(vs, fs) {
    this.id = gl.createProgram();
    
    var vertex = this.AddShader(vs, gl.VERTEX_SHADER);
    var fragment = this.AddShader(fs, gl.FRAGMENT_SHADER);
    
    gl.attachShader(this.id, vertex);
    gl.attachShader(this.id, fragment);
    gl.linkProgram(this.id);
    
    if (!gl.getProgramParameter(this.id, gl.LINK_STATUS)) {
      alert("Could not initialise shaders");
    }

    gl.useProgram(this.id);
    
    this.positionAttribute = gl.getAttribLocation(this.id, "position");
    this.texCoordAttribute = gl.getAttribLocation(this.id, "texCoord");
    this.projectionUniform = gl.getUniformLocation(this.id, "projection");
    this.modelviewUniform = gl.getUniformLocation(this.id, "model");
    this.colourUniform = gl.getUniformLocation(this.id, "colour");
    this.textureUniform = gl.getUniformLocation(this.id, "texture");
}

Shader.prototype.AddShader = function(src, type) {
    var shader = gl.createShader(type);
    gl.shaderSource(shader, src);
    gl.compileShader(shader);
    
    if (!gl.getShaderParameter(shader, gl.COMPILE_STATUS)) {
          console.error(gl.getShaderInfoLog(shader));
          return null;
    }

    return shader;
}

Shader.prototype.Bind = function() {
    gl.useProgram(this.id);
}

Shader.prototype.UniformMatrix = function(pos, val) {
    gl.uniformMatrix4fv(pos, false, val);
}

Shader.prototype.UniformFloat4 = function(pos, val) {
    gl.uniform4f(pos, val[0], val[1], val[2], val[3]);
}


Shader.prototype.UniformTexture = function(pos, val) {
    gl.uniform1i(pos, val);
}
